 /*  Arduino Micro program to run a 3D-printed Joystick with 4 axes, 2 buttons and a hat switch.
 *  
 *  NOTE: This sketch file is for use with Arduino Leonardo and Arduino Micro only. 
 *  Uses Joystick 1.0 library from Matthew Heironimus
 *  
 *  https://github.com/MHeironimus/ArduinoJoystickLibrary/tree/version-1.0
 *  --------------------------------------------------------------------
 */
#include "Joystick.h"

// number of buttons:
int btnCount = 6;
//connected pins
int pins[6] = {0, 1, 7, 8, 10, 11};
int axis[4] = {A0, A1, A2, A3};
//button state at each ime point
int buttonState[6];


void setup() {
  //initialize button pins
  for(int index = 0; index < btnCount;index++){
    pinMode(pins[index], INPUT_PULLUP);
  }

  // Initialize Joystick Library
  Joystick.begin();
}

void loop() {

  // record, whether the hat switch is pressed
  int hatSwitchPressedCount=0;
  
  // Read all pin values (last 4 buttons are hatswitch)!
  for(int index = 0; index < btnCount ;index++){
    int currentButtonState = !digitalRead(pins[index]);
    // Change Joystick buutons only if the state has changed
    if(index < 2){
      if (currentButtonState != buttonState[index]){
        Joystick.setButton(index, currentButtonState);
        buttonState[index] = currentButtonState;
      }
    }else{
        //Record all pressed hat switch buttons
        buttonState[index] = currentButtonState;
        if(buttonState[index]==1) hatSwitchPressedCount++;
    } 
  }

  // position of the hat switch represented as number:
  int hatSwitchState = 0;
  /*   HatSwitch calculation explanation
   *    Buttons:  | Values:  | Two buttons pressed: | Angles:
   *        5     |    1     |         1            |        0
   *              |          |   (1+8)   (1+2)      |    315    45
   *    3       2 | 8     2  |   8           2      |  270        90
   *              |          |   (4+8)   (2+4)      |    225   135
   *        4     |    4     |         4            |       180
   * 
   */
  
  if(hatSwitchPressedCount>0 && hatSwitchPressedCount<3){
    if(buttonState[5]==1) hatSwitchState += 1;
    if (buttonState[4]==1) hatSwitchState += 4;
    if (buttonState[3]==1) hatSwitchState += 8;
    if (buttonState[2]==1) hatSwitchState += 2;

    switch(hatSwitchState){
      case 1: Joystick.setHatSwitch(0,360);
      break;
      case 3: Joystick.setHatSwitch(0,45);
      break;
      case 2: Joystick.setHatSwitch(0,90);
      break;
      case 6: Joystick.setHatSwitch(0,135);
      break;
      case 4: Joystick.setHatSwitch(0,180);
      break;
      case 12: Joystick.setHatSwitch(0,225);
      break;
      case 8: Joystick.setHatSwitch(0,270);
      break;
      case 9: Joystick.setHatSwitch(0,315);
      break;
      default: Joystick.setHatSwitch(0,-1);
    }
  } else Joystick.setHatSwitch(0,-1);

  int x = (int)(analogRead(axis[0])/4-127.5);
  int y = (int)(analogRead(axis[1])/4-127.5);
  int r = (int)(analogRead(axis[2])/4);
  int t = (int)(analogRead(axis[3])/4);
  
  Joystick.setXAxis(x);
  Joystick.setYAxis(y);
  Joystick.setRudder(r);
  Joystick.setThrottle(t);
  
  delay(10);
}
